#!/bin/sh
confdir='/etc/asterisk/lcdialsh/'
CONFFILE="${confdir}LCDial.sh.conf"
ENUMCONFFILE='/etc/asterisk/enum.conf'
#
# Version 1.1: If two providers have the same cost, they will be sorted
#    based on their position in the configuration file. Before they were
#    sorted based on the alphabetical order of their name, which makes
#    little sense and makes it difficult to define preferences among
#    same-cost providers. This change aligns the behaviour of LCDial.sh
#    to LCDC's.
# Version 1.0: added optional 8th parameter to specify script announcing
#    to the caller the cost of the call
# Version 0.8a: tweaked code to extend compatibility with older versions 
#    of ash that supported only expr but not $(( ... )), and reworked 
#    the code that does nothing except defining functions and variables
#    when LCDial.sh is called without parameters ("return" not within
#    a function was frowned upon by bash, and "exit" caused old versions
#    of ash to terminate...)
# Version 0.8:  modified "exclusion" cost from kludgy "9999999" to "!".
#    This also allows mixing in the same file lines with "!"
#    and a numeric value (and identical prefix), and let the
#    numeric line prevail (for how "sort -n" works, that was not
#    the case before: and mixing lines with identical prefix and
#    cost values different but both numeric STILL produces 
#    unpredictable results). 
#    This change breaks (mildly) compatibility with some existing
#    ratefiles, but allows a simplified generation of rate
#    files for "free or nothing" providers (see doc file about
#    "p_" files).
# Version 0.7a:  replaced flaky code based on open_r() and close_r() with
#    safer alternatives 
# Version 0.7:  Added optional "minimum route cost" declaration in first
#    field of /etc/asterisk/lcdialsh/LCDial.sh.conf
#    Added optional 7th parameter to the AGI() call:
#      AGI(LCDial.sh,number,timeout,flags,URL,maxcost,altfilesuffix) 
#    to specify the suffix for an alternate configuration file 
#               (for that call only). The suffix is appended after a '-'
#               (e.g., if CONFFILE='/etc/asterisk/lcdialsh/LCDial.sh.conf'
#               a parameter xyz will instruct the script to use, instead,
#               '/etc/asterisk/lcdialsh/LCDial.sh.conf-xyz'
# Version 0.6a: Fixed bug (missing '-n' option to sort in get_prefcost()) which 
#    affected installations making use of GNU sort rather than 
#    Busybox' sort.
# Version 0.6:  Fixed incompatibilities with bash. Now /bin/sh may be
#    either ash or bash.
# Version 0.5:  Called with no arguments it silently exit. Useful to
#    other scripts to source function definitions.
# Version 0.4b: NOANSWER after less than 5 sec of dialing treated as CHANUNAVAIL
#               Added fwdout as provider
#    Corrected format of Dial channel obtained through ENUM lookups
# Version 0.4a: corrected minor bug in ratefile
# Version 0.4 - 12 Sep 2005
# NEW: lines in rate files may now be commented by ';' or '#'
# NEW: lines in configuration file may take a third field: 
#      number2postdialdigits (see doc file)
# Version 0.3 - 11 Aug 2005
# NEW: lines in rate files may now contain start time, end time and 
#      day of the week when the rate applies (see doc file)
#
# Copyright (C) Enzo Michelangeli 2005
#
# --------- no user-configurable parameters below this line
#

lf="
"
tab=$(echo '_' | tr '_' '\t')  # sigh...

do_dial() {
  local dialstarttime=$(date '+%s')
  sh_agi_cmd EXEC Dial \""$1"\"
  # sh_agi_log "After Dial: sh_agi_response=$sh_agi_response, sh_agi_result=$sh_agi_result"
  sh_agi_getvariable DIALSTATUS
  sh_agi_log "$(expr $(date '+%s') - $dialstarttime) seconds after Dial: DIALSTATUS=$sh_agi_DIALSTATUS" "1"
  case "$sh_agi_DIALSTATUS" in 
  CONGESTION|CHANUNAVAIL)
    # continue trying next channel
  ;;
  NOANSWER)
    # to be "true" (noone picked up the phone), NOANSWER must arrive 
    # more than 5 seconds after starting dialing. Otherwise it's likely
    # to be a CHANUNAVAIL misrepresented as NOANSWER.
    if [ $(expr $(date '+%s') - $dialstarttime) -gt 5 ]; then exit 0; fi
  ;;
  ANSWER)
    # to be "true" (complete with conversation), ANSWER must arrive 
    # more than 20 seconds after starting dialing. Otherwise it's likely
    # to be a CHANUNAVAIL misrepresented as NOANSWER.
    if [ $(expr $(date '+%s') - $dialstarttime) -gt 20 ]; then exit 0; fi
  ;;
  BUSY|CANCEL)
    exit 0;
  ;;
  *)
    sh_agi_log "*** After Dial: unexpected DIALSTATUS=$sh_agi_DIALSTATUS" "1"
    exit 0;  # unexpected
  ;;
  esac
}

get_costs() {
  local ratefile="$1"
  local now=$(date '+%H%M')  # current localtime expressed as hhmm
  local starttime
  local endtime
  local today=$(date '+%w')  # current day of week expressed 0-6 (0 = Sun)
  local weekday
  sed -e "s/[;#].*//; s/\$/_$callednumber/; s/^[^-]*-[^!0-9-]\+\([^_]\+\).*/-,\1/;  s/^[^0-9]*\([0-9]\+\)[[:space:]]\+\([^[:space:]_]\+\)\([^_]*\)_\1[0-9]*/\1,\2\3/;/_/d"  \
  "$ratefile" | while read prefix cost starttime endtime weekday; do
    # check if there is a range, and in that case if we are within it
    case "_$starttime" in

    _)
      echo "$prefix $cost"
      ;;
    *)
      case "_$endtime" in
      _)
        endtime='2359'
      ;;
      *)
        # check if there is also a list of weekdays of validity
        case "_$weekday" in
        _)
        ;;  # not present, fine
        *)  # if the weekday parameter is not blank
          case "_$weekday" in
          *"$today"*)
          ;;  # ...and does not contain today:
          *)
            continue  # skip this record
          ;;
          esac
        ;;
        esac
      ;;
      esac
      if [ "$starttime" -le "$endtime" ]; then
        if [ "$now" -ge "$starttime" -a "$now" -le "$endtime" ]; then
          echo "$prefix $cost"
        fi
      else
        if [ "$now" -ge "$starttime" -o "$now" -le "$endtime" ]; then
          echo "$prefix $cost"
        fi
      fi            
    ;;
    esac
  done 
}

# find cost per minute in the specified ratefile for the best match of
# the global variable $callednumber. Returns the relative cost in
# millicents per minute into the global variable $cost
#
find_bestmatch_cost() {    # ratefile
  local saveIFS="$IFS"
  IFS=" $lf$tab\","  # For CSV files w/o embedded spaces. NOTE: it must start with ' ' (see ash docs)
  local bestmatch=""
  local prefix
  local prefcost
  local ratefile=$1
  if [ -f "$ratefile" ]; then
    # look for records where the prefix is a substring of $callednumber
    # and formats them into the form "prefix,cost"
    # then sort the latter and pick the last, which is the longest
    # (also because ',' lexicographically precedes [0-9]...
    # sed hackery: kids, don't try this at home!
    prefcost="$(get_costs $ratefile | sort -n | tail -1)"
    case "_$prefcost" in
    _)
      unset cost
    ;;
    *)
      prefix=$(echo "$prefcost" | cut -d " " -f 1)
      cost=$(echo "$prefcost" | cut -d " " -f 2)
    ;;
    esac
  else
    sh_agi_log "*** Ratefile $ratefile does not exist!" "1"
  fi

#  case _$prefix in
#  _)
#    sh_agi_log "'>>>>>'for $callednumber $ratefile has NO routes"
#  ;;
#  *)
#    sh_agi_log "'>>>>>'for $callednumber $ratefile has prefix $prefix at $cost millicents/minute"
#  ;;
#  esac

  IFS="$saveIFS"
}

# $1 is set to the Dial called number (guaranteed to contain only dec. digits)
# $2 is set to the Dial timeout (or unset)
# $3 is set to the Dial options string (or unset)
# $4 is set to the Dial URL string (or unset)
# $5 is set to the max cost in millicents (or unset)

# If called with no arguments, it silently exits. This allows other scripts
# to source the function definitions...

case _"$1" in
_)
;;
*)
  # only if called with at least one parameter...
  case _"$7" in
  _)
  ;;
  *)
    ANNOUNCE="$7"
  ;;
  esac
  
  case _"$6" in
  _)
  ;;
  *)
    CONFFILE="$CONFFILE-$6"
  ;;
  esac

  agidir=$(echo "$0"|sed -e "s!$(basename $0)!!") # no dirname here...
  
  if [ "_$agidir" = _ ]; then
  	agidir='/var/lib/asterisk/agi-bin/'
  fi

  . ${agidir}sh-agi.inc

  callednumber=$(echo "$1" | sed -e 's/[^0-9]//g') # remove non-digits
  case _"$5" in 
  _) 
    costlimit=9999999
  ;;
  *)
    costlimit="$(expr $5 \+ 1)"
  ;;
  esac

  opts="$3"
  timeout="$2"
  dialurl="$4"

  # Try all the VoIP termination providers serving that number, starting
  # with the cheapest for it, and dial only if the cost per minute
  # is less or equal to $5.
  # ENUM is an "implicit" free provider, and the existence of service
  # plus the URI rule is given by listing the NAPTR records for the
  # reversed number

  saveIFS="$IFS"
  IFS=" $lf$tab"  # A sane value. NOTE: it must start with ' ' (see ash docs)

  # Automatically rebuild, "make"-style, stale ratefiles based on "p_" and "c_"
  # description files

  curdir=$(pwd)
  cd ${confdir}rates
  for provfile in p_* ; do
    # (see if the ratefile is newer than provider file and all country files
    # referenced by the provider file)
    provname=$(echo "$provfile" | sed 's/^p_//')
    newest=$(ls -t $(sed 's/^/c_/' "$provfile") "$provfile" "$provname" | sed q)
    if [ "$newest" != "$provname" ]; then
      sh_agi_log "$provname out of date, rebuilding..."
      rm "$provname"
      for fc in $(cat "$provfile"); do
        echo  >>"$provname" "; $fc"
        sed >>"$provname" "s/[#;].*//;s/[[:space:]]\+\$//;s/[[:space:]]\+/$tab/;/^\$/d" < "c_$fc"
      done
      sh_agi_log "...done."
    fi
  done
  cd $curdir 

  line=0
  providers_list=""

  if [ -r "$ENUMCONFFILE" ]; then
  if dig=$(which dig 2>/dev/null); then 
    position=10000
    IFS="$lf"
    for line in $(cat "$ENUMCONFFILE"); do
      IFS=" $lf$tab"
      case "_$line" in
      _)
        continue
      ;;
      esac
      ENUMDOM=$(echo "$line" \
      | egrep '[sS][eE][aA][rR][cC][hH][ ]*=>[ ]*' \
      | sed 's/[ ]*[sS][eE][aA][rR][cC][hH][ ]*=>[ ]*\([^ ]\+\)/\1/')
      case "_$ENUMDOM" in
      _)
        continue
      ;;
      esac
      # start with an ENUM lookup for all the domains in /etc/asterisk/enum.conf
      CNT=0
      # convert "12345" into "5.4.3.2.1."
      REVNUM=$(echo "$callednumber" \
      | sed '/\n/!G;s/\(.\)\(.*\n\)/&\2\1./;/..*\n/D;s/.//')
      # get the NAPTR records (if any) discarding URI rules not for SIP or IAX(2)
      HOSTOUTPUT=$($dig -t naptr "${REVNUM}${ENUMDOM}." \
      | egrep '"*u"*[ ]+"*E2U\+' \
      | sed 's/.*NAPTR[^0-9]*\(.*\)/\1/' \
      | sed 's/\([0-9]\+\)[^0-9]\+\([0-9]\+\)[ ]\+[^ ]\+[ ]\+[^ ]\+[ ]\+\([^ ]\+\)/\1_\2%\3/' \
      | sed '/\(sip:\|iax:\|iax2:\)/!d;s/\\\\+//;s/(/\\(/;s/)/\\)/' \
      | sed 's/\\\\/\\/;s/"\(.*\)".*/s\1/;s/\^//g;s/\$//g')

      for URIrule in $HOSTOUTPUT; do
        ORDER_PREF=$(echo "$URIrule" | cut -d '%' -f 1)
        URIrule=$(echo "$URIrule" | cut -d '%' -f 2)
        URI="$(echo "$callednumber" | sed -e "$URIrule")"
        sh_agi_log "=== found ENUM route with ${ENUMDOM}: $URI"
        # convert from IETF URI to Asterisk URI (sip/number@domain or iax2/user@domain/number)
        URI="$(echo "$URI" \
        | sed -e 's!\([^:]\+\):\([^@]*\)\@\(.\+\)!\1/\2@\3!')"
        # create a dummy provider domain_priority_counter
        provider=$(echo "${ENUMDOM}_${ORDER_PREF}_${CNT}" | sed -e 's/\./_/g')
        # and place it in a ENUMnn_record with cost=0

        eval "${provider}_URI"=\"$URI\"
        eval "${provider}_cost"=\"0\"
        eval "${provider}_record"=\""0 $position $provider"\"
        position=$(expr $position \+ 1)
        #append ENUMnn to providers_list 
        providers_list="$providers_list $provider"
        CNT=$(expr $CNT \+ 1)
      done
    done
  else
    # dig is not avalable, so we can't use it to get
    # all the NAPTR records for SIP and IAX rules. So,
    # let's fall back to the less powerful EnumLookup Asterisk app
    # and try to dial the single URI returned by it:

    sh_agi_cmd EXEC EnumLookup "$callednumber"
    #sh_agi_log "response: $sh_agi_response result: $sh_agi_result"

    sh_agi_getvariable ENUM
    # if set, the ENUM query was successful. The sh_agi_ENUM variable
    # will then be set to something like:
    # IAX2/guest@em.no-ip.com/8829900152401

    sh_agi_log "ENUM var: $sh_agi_ENUM"

    case "_$sh_agi_ENUM" in
    _IAX* | _SIP*) 
      do_dial "$sh_agi_ENUM|$timeout|$opts|$dialurl"
      case "$sh_agi_result" in
      0*)
        return 0
      ;;
      esac
      ;;
    esac
  fi
  fi
  # now scan the actual providers referenced by $CONFFILE

  confline=0
  IFS="$lf"
  for line in $(cat "$CONFFILE"); do
    IFS=" $lf$tab"
    #read -r provider ratefile URIrule postdialrule
    set $line
    provider=$1
    case "_$provider" in
    _\;*|_)
      confline=$(expr $confline \+ 1)
      continue  # comment or empty
    ;;
    esac

    # for providers with a ">minval" qualifier, skip if minval >= costlimit
    minval=$(echo "$provider" | sed -n -e 's![^>]*>\([0-9]\+\)!\1!p')
    case "_$minval" in
    _)
      minval=0
    ;;
    *)
      minval=$(expr $minval \+ 1)
    ;;
    esac
    provider=$(echo "$provider" | sed -e 's![>].*!!')
    if [ "$minval" -ge "$costlimit" ]; then 
      confline=$(expr $confline \+ 1)
      continue 
    fi
    # sh_agi_log "--- $provider acceptable: minval $minval lower than costlimit $costlimit"
    unset minval

    ratefile=$2
    URIrule=$3
    postdialrule=$4

    # sh_agi_log "confline:$confline: provider:$provider: ratefile:$ratefile: URIrule:$URIrule: postdialrule:$postdialrule" "" 
    case _"$URIrule" in
    _)
      sh_agi_log "*** $CONFFILE:$confline: malformed line (less than three parameters): ignored" "1"
      confline=$(expr $confline \+ 1)
      continue;
    ;;
    esac

    eval provider_defined=\$${provider}_defined
    case _$provider_defined in
    _)
      eval ${provider}_defined=yes
      cost=
      find_bestmatch_cost "$confdir$ratefile" # <<<<<<<<<<<<<<<<<<
      case "_$cost" in
      _)
      ;;
      _!)  # '!' is the "exclusion" marker
      ;;
      *)
        if [ "$cost" -lt "$costlimit" ]; then 
    sh_agi_log "=== found rate ($cost) < limit $costlimit with $provider"
    URI="$(echo $callednumber | sed -e $URIrule)"

    case _$postdialrule in
    _) 
    ;;
    *)
      postdial="$(echo $callednumber | sed -e $postdialrule)"
      eval "${provider}_postdial"=\"$postdial\"
    ;;
    esac

    # store temporarily URIs and (provider, cost) pairs in 
    # kind of associative arrays
    eval "${provider}_URI"=\"$URI\"
    eval "${provider}_cost"=\"$cost\"
    position=$(expr $confline \+ 20000)
    eval "${provider}_record"=\""$cost $position $provider"\"
    providers_list="$providers_list $provider"
        fi
      ;;
      esac
    ;;
    *)
      sh_agi_log "*** $CONFFILE:$confline: redefinition of provider $par1: ignored" "1"
    ;;
    esac
    confline=$(expr $confline \+ 1)
  done
  IFS="$saveIFS"

  case "_$providers_list" in
  _)
    sh_agi_log "**** Sorry! No provider can deliver this call ****"
  ;;
  *)
    IFS=" $tab"  # keep \n as literal here!
    sorted_providers_list=$( (for provider in $providers_list ; do
      eval echo "\$${provider}_record"
      done
    ) | sort -n | while read cost position provider ; do
      echo $provider 
    done )

    IFS=" $tab$cr$lf"  # treat \n again as separator expanding $sorted_providers_list
    # now providers list is sorted on ascending cost  
    for provider in $sorted_providers_list ; do
      eval URI=\""\$${provider}_URI"\"
      eval cost=\""\$${provider}_cost"\"
      eval postdial=\""\$${provider}_postdial"\"
      case _"$postdial" in
      _)
      ;;
      *)
        opts=$(echo "$opts" | sed -e "s/D(.*)//;s/$/D($postdial)/")
      ;;
      esac
      sh_agi_log "==\> Dial command: $URI|$timeout|$opts|$dialurl - cost: $cost millicents/min"
     
      case "_$ANNOUNCE" in
      _|_/*)
      ;;
      *)
        ANNOUNCE="${agidir}$ANNOUNCE" # if it doesn't start by '/', it's in AGI dir
      ;;
      esac
      
      if [ "_$ANNOUNCE" != "_" -a -f "$ANNOUNCE" ]; then 
        sh_agi_log "==\> calling: . $ANNOUNCE $cost"
        . $ANNOUNCE "$cost"
      fi
            
      # if successful, do_dial does not return.
      do_dial "$URI|$timeout|$opts|$dialurl"
    done  
    IFS="$saveIFS"
  ;;
  esac

  sh_agi_cmd HANGUP
;;
esac
